@@ -22,7 +22,8 @@ from subprocess import Popen, PIPE, STDOUT
 import inspect
 from collections import deque
 
-from microarch import get_all_data, get_all_data_from_marchid, is_superset_of, set_diff, extensions_to_names, get_compiler_flags_arch_id
+from microarch import get_all_data, get_all_data_from_marchid, is_superset_of, set_diff, extensions_to_names, get_compiler_flags_arch_id, level3_on, encode_extensions
+
 
 DEFAULT_ORGANIZATION_NAME = 'k-nuth'
 DEFAULT_LOGIN_USERNAME = 'fpelliccioni'
@@ -526,6 +527,31 @@ def get_builder(recipe_dir, args=None):
 
 # --------------------------------------------
 
+def handle_microarchs(opt_name, microarchs, filtered_builds, settings, options, env_vars, build_requires):
+    microarchs = list(set(microarchs))
+
+    for ma in microarchs:
+        opts_copy = copy.deepcopy(options)
+        opts_copy[opt_name] = ma
+        filtered_builds.append([settings, opts_copy, env_vars, build_requires])
+
+def get_base_march_ids():
+    # return ['4fZKi37a595hP']        # haswell
+    level3_exts = level3_on()
+    level3_marchid = encode_extensions(level3_exts)
+    return [level3_marchid]
+
+def filter_marchs_tests(name, builds, test_options, march_opt=None):
+    if march_opt is None:
+        # march_opt = "%s:microarchitecture" % name
+        march_opt = "%s:march_id" % name
+
+    for b in builds:
+        options = b[1]
+        if options[march_opt] != "x86-64":
+            for to in test_options:
+                options[to] = "False"
+
 # usage:
 # conan install package -o march_id=? -o march_strategy=quick (default)
 # conan install package -o march_id=? -o march_strategy=optimized
@@ -538,7 +564,6 @@ def get_builder(recipe_dir, args=None):
 # -o march_from=cpu
 # -o march_from=both
 
-
 def march_conan_manip(conanobj):
     if conanobj.settings.arch != "x86_64":
         return (None, None, None, None)
@@ -550,7 +575,6 @@ def march_conan_manip(conanobj):
         return (None, None, None, None)
 
     conanobj.march_from_cpuid = True
-    march_from_str = 'taken from cpuid'
     march_id = None
     march_names = None
     march_flags = None
@@ -587,7 +611,6 @@ def march_conan_manip(conanobj):
     else:
         march_id = str(conanobj.options.march_id)
         conanobj.march_from_cpuid = False
-        march_from_str = 'user defined'
 
         conanobj.march_data = get_all_data_from_marchid(
                                         march_id,
@@ -608,7 +631,10 @@ def march_conan_manip(conanobj):
         #     march_flags = conanobj.march_data['comp_flags']
         # elif conanobj.options.march_strategy == "download_if_possible":
 
-    conanobj.output.info("Detected microarchitecture ID (%s): %s" % (march_from_str, march_id))
+    if conanobj.march_from_cpuid:
+        conanobj.output.info(f"Detected microarchitecture ID: {march_id}")
+    else:
+        conanobj.output.info(f"User-defined microarchitecture ID: {march_id}")
 
     return (march_id, march_names, march_flags)
 
@@ -623,8 +649,8 @@ def pass_march_to_compiler(conanobj, cmake):
                             float(str(conanobj.settings.compiler.version)))
 
     conanobj.output.info("Compiler flags: %s" % flags)
-    conanobj.output.info("Prev CONAN_CXX_FLAGS: %s" % cmake.definitions.get("CONAN_CXX_FLAGS", ""))
-    conanobj.output.info("Prev CONAN_C_FLAGS: %s" % cmake.definitions.get("CONAN_C_FLAGS", ""))
+    # conanobj.output.info("Prev CONAN_CXX_FLAGS: %s" % cmake.definitions.get("CONAN_CXX_FLAGS", ""))
+    # conanobj.output.info("Prev CONAN_C_FLAGS: %s" % cmake.definitions.get("CONAN_C_FLAGS", ""))
 
     cmake.definitions["CONAN_CXX_FLAGS"] = cmake.definitions.get("CONAN_CXX_FLAGS", "") + " " + flags
     cmake.definitions["CONAN_C_FLAGS"] = cmake.definitions.get("CONAN_C_FLAGS", "") + " " + flags
@@ -685,9 +711,8 @@ class KnuthConanFile(ConanFile):
     #         self.output.warn("**** The recipe does not implement the march_id option. ****")
 
     def validate(self):
-
-        self.output.info(f"validate() self.march_data: {self.march_data}")
-        self.output.info(f"validate() self.march_from_cpuid: {self.march_from_cpuid}")
+        # self.output.info(f"validate() self.march_data: {self.march_data}")
+        # self.output.info(f"validate() self.march_from_cpuid: {self.march_from_cpuid}")
 
         # ConanFile.validate(self)
         if self.conan_req_version != None and Version(conan_version) < Version(self.conan_req_version):
@@ -707,19 +732,19 @@ class KnuthConanFile(ConanFile):
                     if not is_superset_of(exts, level3_exts):
                         exts_diff = set_diff(level3_exts, exts)
                         exts_names = extensions_to_names(exts_diff)
-                        exts_str = " ".join(exts_names)
-                        raise ConanInvalidConfiguration(f"The detected micro-architecture of your system is not compatible with x86-64-v3 (Check https://en.wikipedia.org/wiki/X86-64#Microarchitecture_levels).\nThe following extensions are not supported by your system: {exts_str}.\nThis error is generated because you chose march_strategy = download_or_fail.")
+                        exts_str = ", ".join(exts_names)
+                        raise ConanInvalidConfiguration(f"The detected microarchitecture of your system is not compatible with x86-64-v3 (Check https://en.wikipedia.org/wiki/X86-64#Microarchitecture_levels).\nThe following extensions are not supported by your system: {exts_str}.\nThis error is generated because you chose march_strategy = download_or_fail.")
             else:
-                if self.march_data['user_marchid_valid']:
-                    raise ConanInvalidConfiguration(f"{self.options.get_safe("march_id")} is not a valid micro-architecture id (march_id option).")
+                if not self.march_data['user_marchid_valid']:
+                    raise ConanInvalidConfiguration(f"{self.options.get_safe('march_id')} is not a valid microarchitecture id (march_id option).")
 
                 exts = self.march_data['user_exts']
                 exts_filtered = self.march_data['user_exts_filtered']
                 if not is_superset_of(exts_filtered, exts):
                     exts_diff = set_diff(exts, exts_filtered)
                     exts_names = extensions_to_names(exts_diff)
-                    exts_str = " ".join(exts_names)
-                    raise ConanInvalidConfiguration(f"{self.options.get_safe("march_id")} is not compatible with your compiler.\nThe following extensions are not supported by your compiler: {exts_str}.")
+                    exts_str = ", ".join(exts_names)
+                    raise ConanInvalidConfiguration(f"{self.options.get_safe('march_id')} is not compatible with your compiler.\nThe following extensions are not supported by your compiler: {exts_str}.")
 
 
     def configure(self, pure_c=False):
